/*
 * matutil.c
 */

#include "matutil.h"

static void error(message)
char *message;
{
    fprintf(stderr, "\n%s\n", message);
    exit(0);
}

vector newvec(n)
int n;
{
    return (vector)malloc(sizeof(SCALAR) * n);
}

matrix newmat(nrow, ncol)
int nrow, ncol;
{
    int i;
    matrix a;

    a = (matrix)malloc((nrow + 1) * sizeof(void *));
    if (a == NULL) return NULL;
    for (i = 0; i < nrow; i++) {
        a[i] = (SCALAR *)malloc(sizeof(SCALAR) * ncol);
        if (a[i] == NULL) {
            while (--i >= 0) free(a[i]);
            free(a);
            return NULL;
        }
    }
    a[nrow] = NULL;
    return a;
}

vector new_vector(n)
int n;
{
    vector v;

    v = newvec(n);
    if (v == NULL) error("ΰ­Ǥ");
    return v;
}

matrix new_matrix(nrow, ncol)
int nrow, ncol;
{
    matrix a;

    a = newmat(nrow, ncol);
    if (a == NULL) error("ΰ­Ǥ");
    return a;
}

void free_vector(v)
vector v;
{
    free(v);
}

void free_matrix(a)
matrix a;
{
    matrix b;

    b = a;
    while (*b != NULL) free(*b++);
    free(a);
}

double innerproduct(n, u, v)
int n;
vector u, v;
{
    int i, n5;
    double s;

    s = 0;
    n5 = n % 5;
    for (i = 0; i < n5; i++)
        s += u[i]*v[i];
    for (i = n5; i < n; i += 5)
        s += u[i]*v[i] + u[i+1]*v[i+1] + u[i+2]*v[i+2] +
            u[i+3]*v[i+3] + u[i+4]*v[i+4];
    return s;
}

void vecprint(v, n, perline, format)
vector v;
int n, perline;
char *format;
{
    int j, k;

    k = 0;
    for(j = 0; j < n; j++) {
        printf(format, v[j]);
        if (++k >= perline) {
            k = 0;
            printf("\n");
        }
    }
    if (k != 0)
        printf("\n");
}

void matprint(a, ncol, perline, format)
matrix a;
int ncol, perline;
char *format;
{
    int i;

    for (i = 0; a[i] != NULL; i++) {
        vecprint(a[i], ncol, perline, format);
        if (ncol > perline)
            printf("\n");
    }
}
